import type { Pupil } from 'medama';
import { useContext, useMemo } from 'react';
import { pupilProviderContext, type PupilProviderProps } from './ProvideMedama';

export type MedamaReactHookOptions<State extends object> =
  | { pupil: Pupil<State> }
  | { id: string | number | symbol };

export const useMedama = <State extends object>(
  options: Partial<MedamaReactHookOptions<State>> = {}
): Pupil<State> => {
  const contextValue = useContext(pupilProviderContext) as PupilProviderProps<State>;
  const pupilBeenProvided = 'pupil' in options ? options.pupil : undefined;

  const pupil = useMemo(() => {
    if (pupilBeenProvided) return pupilBeenProvided;

    const { pupil: pupilFromContext } =
      (!('id' in options) || options.id == null || options.id === contextValue.id
        ? contextValue
        : contextValue.lookUp?.(options.id)) ?? {};

    return pupilFromContext;
  }, ['id' in options && options, pupilBeenProvided, contextValue.pupil, contextValue.id]);

  if (pupil == null)
    throw new Error(
      'Medama Error: ProvideMedama component must be initialized up in component tree'
    );

  return pupil;
};
